# ----------------------------------------------------------------------------
# pyglet
# Copyright (c) 2006-2008 Alex Holkner
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions 
# are met:
#
#  * Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#  * Redistributions in binary form must reproduce the above copyright 
#    notice, this list of conditions and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
#  * Neither the name of pyglet nor the names of its
#    contributors may be used to endorse or promote products
#    derived from this software without specific prior written
#    permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
# ----------------------------------------------------------------------------
'''Wrapper for openal'''

__docformat__ =  'restructuredtext'
__version__ = '$Id$'

import ctypes
from ctypes import *
import sys

import pyglet.lib

_lib = pyglet.lib.load_library('openal', win32='openal32',
        framework='/System/Library/Frameworks/OpenAL.framework')

_int_types = (c_int16, c_int32)
if hasattr(ctypes, 'c_int64'):
    # Some builds of ctypes apparently do not have c_int64
    # defined; it's a pretty good bet that these builds do not
    # have 64-bit pointers.
    _int_types += (ctypes.c_int64,)
for t in _int_types:
    if sizeof(t) == sizeof(c_size_t):
        c_ptrdiff_t = t

class c_void(Structure):
    # c_void_p is a buggy return type, converting to int, so
    # POINTER(None) == c_void_p is actually written as
    # POINTER(c_void), so it can be treated as a real pointer.
    _fields_ = [('dummy', c_int)]





ALsizei = c_int
ALfloat = c_float
ALenum = c_int
ALuint = c_uint
ALint = c_int
ALboolean = c_int
AL_TRUE = 1
AL_FALSE = 0

ALC_EXT_EFX_NAME = "ALC_EXT_EFX"

ALC_EFX_MAJOR_VERSION = 131073
ALC_EFX_MINOR_VERSION = 131074
ALC_MAX_AUXILIARY_SENDS = 131075

AL_METERS_PER_UNIT = 131076

AL_DIRECT_FILTER = 131077
AL_AUXILIARY_SEND_FILTER = 131078
AL_AIR_ABSORPTION_FACTOR = 131079
AL_ROOM_ROLLOFF_FACTOR = 131080
AL_CONE_OUTER_GAINHF = 131081
AL_DIRECT_FILTER_GAINHF_AUTO = 131082
AL_AUXILIARY_SEND_FILTER_GAIN_AUTO = 131083
AL_AUXILIARY_SEND_FILTER_GAINHF_AUTO = 131084

AL_REVERB_DENSITY = 1
AL_REVERB_DIFFUSION = 2
AL_REVERB_GAIN = 3
AL_REVERB_GAINHF = 4
AL_REVERB_DECAY_TIME = 5
AL_REVERB_DECAY_HFRATIO = 6
AL_REVERB_REFLECTIONS_GAIN = 7
AL_REVERB_REFLECTIONS_DELAY = 8
AL_REVERB_LATE_REVERB_GAIN = 9
AL_REVERB_LATE_REVERB_DELAY = 10
AL_REVERB_AIR_ABSORPTION_GAINHF = 11
AL_REVERB_ROOM_ROLLOFF_FACTOR = 12
AL_REVERB_DECAY_HFLIMIT = 13

AL_EAXREVERB_DENSITY = 1
AL_EAXREVERB_DIFFUSION = 2
AL_EAXREVERB_GAIN = 3
AL_EAXREVERB_GAINHF = 4
AL_EAXREVERB_GAINLF = 5
AL_EAXREVERB_DECAY_TIME = 6
AL_EAXREVERB_DECAY_HFRATIO = 7
AL_EAXREVERB_DECAY_LFRATIO = 8
AL_EAXREVERB_REFLECTIONS_GAIN = 9
AL_EAXREVERB_REFLECTIONS_DELAY = 10
AL_EAXREVERB_REFLECTIONS_PAN = 11
AL_EAXREVERB_LATE_REVERB_GAIN = 12
AL_EAXREVERB_LATE_REVERB_DELAY = 13
AL_EAXREVERB_LATE_REVERB_PAN = 14
AL_EAXREVERB_ECHO_TIME = 15
AL_EAXREVERB_ECHO_DEPTH = 16
AL_EAXREVERB_MODULATION_TIME = 17
AL_EAXREVERB_MODULATION_DEPTH = 18
AL_EAXREVERB_AIR_ABSORPTION_GAINHF = 19
AL_EAXREVERB_HFREFERENCE = 20
AL_EAXREVERB_LFREFERENCE = 21
AL_EAXREVERB_ROOM_ROLLOFF_FACTOR = 22
AL_EAXREVERB_DECAY_HFLIMIT = 23

AL_CHORUS_WAVEFORM = 1
AL_CHORUS_PHASE = 2
AL_CHORUS_RATE = 3
AL_CHORUS_DEPTH = 4
AL_CHORUS_FEEDBACK = 5
AL_CHORUS_DELAY = 6

AL_DISTORTION_EDGE = 1
AL_DISTORTION_GAIN = 2
AL_DISTORTION_LOWPASS_CUTOFF = 3
AL_DISTORTION_EQCENTER = 4
AL_DISTORTION_EQBANDWIDTH = 5

AL_ECHO_DELAY = 1
AL_ECHO_LRDELAY = 2
AL_ECHO_DAMPING = 3
AL_ECHO_FEEDBACK = 4
AL_ECHO_SPREAD = 5

AL_FLANGER_WAVEFORM = 1
AL_FLANGER_PHASE = 2
AL_FLANGER_RATE = 3
AL_FLANGER_DEPTH = 4
AL_FLANGER_FEEDBACK = 5
AL_FLANGER_DELAY = 6

AL_FREQUENCY_SHIFTER_FREQUENCY = 1
AL_FREQUENCY_SHIFTER_LEFT_DIRECTION = 2
AL_FREQUENCY_SHIFTER_RIGHT_DIRECTION = 3

AL_VOCAL_MORPHER_PHONEMEA = 1
AL_VOCAL_MORPHER_PHONEMEA_COARSE_TUNING = 2
AL_VOCAL_MORPHER_PHONEMEB = 3
AL_VOCAL_MORPHER_PHONEMEB_COARSE_TUNING = 4
AL_VOCAL_MORPHER_WAVEFORM = 5
AL_VOCAL_MORPHER_RATE = 6

AL_PITCH_SHIFTER_COARSE_TUNE = 1
AL_PITCH_SHIFTER_FINE_TUNE = 2

AL_RING_MODULATOR_FREQUENCY = 1
AL_RING_MODULATOR_HIGHPASS_CUTOFF = 2
AL_RING_MODULATOR_WAVEFORM = 3

AL_AUTOWAH_ATTACK_TIME = 1
AL_AUTOWAH_RELEASE_TIME = 2
AL_AUTOWAH_RESONANCE = 3
AL_AUTOWAH_PEAK_GAIN = 4

AL_COMPRESSOR_ONOFF = 1

AL_EQUALIZER_LOW_GAIN = 1
AL_EQUALIZER_LOW_CUTOFF = 2
AL_EQUALIZER_MID1_GAIN = 3
AL_EQUALIZER_MID1_CENTER = 4
AL_EQUALIZER_MID1_WIDTH = 5
AL_EQUALIZER_MID2_GAIN = 6
AL_EQUALIZER_MID2_CENTER = 7
AL_EQUALIZER_MID2_WIDTH = 8
AL_EQUALIZER_HIGH_GAIN = 9
AL_EQUALIZER_HIGH_CUTOFF = 10

AL_EFFECT_FIRST_PARAMETER = 0
AL_EFFECT_LAST_PARAMETER = 32768
AL_EFFECT_TYPE = 32769

AL_EFFECT_NULL = 0
AL_EFFECT_REVERB = 1
AL_EFFECT_CHORUS = 2
AL_EFFECT_DISTORTION = 3
AL_EFFECT_ECHO = 4
AL_EFFECT_FLANGER = 5
AL_EFFECT_FREQUENCY_SHIFTER = 6
AL_EFFECT_VOCAL_MORPHER = 7
AL_EFFECT_PITCH_SHIFTER = 8
AL_EFFECT_RING_MODULATOR = 9
AL_EFFECT_AUTOWAH = 10
AL_EFFECT_COMPRESSOR = 11
AL_EFFECT_EQUALIZER = 12
AL_EFFECT_EAXREVERB = 32768

AL_EFFECTSLOT_EFFECT = 1
AL_EFFECTSLOT_GAIN = 2
AL_EFFECTSLOT_AUXILIARY_SEND_AUTO = 3

AL_EFFECTSLOT_NULL = 0

AL_LOWPASS_GAIN = 1
AL_LOWPASS_GAINHF = 2

AL_HIGHPASS_GAIN = 1
AL_HIGHPASS_GAINLF = 2

AL_BANDPASS_GAIN = 1
AL_BANDPASS_GAINLF = 2
AL_BANDPASS_GAINHF = 3

AL_FILTER_FIRST_PARAMETER = 0
AL_FILTER_LAST_PARAMETER = 32768
AL_FILTER_TYPE = 32769

AL_FILTER_NULL = 0
AL_FILTER_LOWPASS = 1
AL_FILTER_HIGHPASS = 2
AL_FILTER_BANDPASS = 3

AL_LOWPASS_MIN_GAIN = 0.0
AL_LOWPASS_MAX_GAIN = 1.0
AL_LOWPASS_DEFAULT_GAIN = 1.0

AL_LOWPASS_MIN_GAINHF = 0.0
AL_LOWPASS_MAX_GAINHF = 1.0
AL_LOWPASS_DEFAULT_GAINHF = 1.0

AL_HIGHPASS_MIN_GAIN = 0.0
AL_HIGHPASS_MAX_GAIN = 1.0
AL_HIGHPASS_DEFAULT_GAIN = 1.0

AL_HIGHPASS_MIN_GAINLF = 0.0
AL_HIGHPASS_MAX_GAINLF = 1.0
AL_HIGHPASS_DEFAULT_GAINLF = 1.0

AL_BANDPASS_MIN_GAIN = 0.0
AL_BANDPASS_MAX_GAIN = 1.0
AL_BANDPASS_DEFAULT_GAIN = 1.0

AL_BANDPASS_MIN_GAINHF = 0.0
AL_BANDPASS_MAX_GAINHF = 1.0
AL_BANDPASS_DEFAULT_GAINHF = 1.0

AL_BANDPASS_MIN_GAINLF = 0.0
AL_BANDPASS_MAX_GAINLF = 1.0
AL_BANDPASS_DEFAULT_GAINLF = 1.0

AL_REVERB_MIN_DENSITY = 0.0
AL_REVERB_MAX_DENSITY = 1.0
AL_REVERB_DEFAULT_DENSITY = 1.0

AL_REVERB_MIN_DIFFUSION = 0.0
AL_REVERB_MAX_DIFFUSION = 1.0
AL_REVERB_DEFAULT_DIFFUSION = 1.0

AL_REVERB_MIN_GAIN = 0.0
AL_REVERB_MAX_GAIN = 1.0
AL_REVERB_DEFAULT_GAIN = 0.32

AL_REVERB_MIN_GAINHF = 0.0
AL_REVERB_MAX_GAINHF = 1.0
AL_REVERB_DEFAULT_GAINHF = 0.89

AL_REVERB_MIN_DECAY_TIME = 0.1
AL_REVERB_MAX_DECAY_TIME = 20.0
AL_REVERB_DEFAULT_DECAY_TIME = 1.49

AL_REVERB_MIN_DECAY_HFRATIO = 0.1
AL_REVERB_MAX_DECAY_HFRATIO = 2.0
AL_REVERB_DEFAULT_DECAY_HFRATIO = 0.83

AL_REVERB_MIN_REFLECTIONS_GAIN = 0.0
AL_REVERB_MAX_REFLECTIONS_GAIN = 3.16
AL_REVERB_DEFAULT_REFLECTIONS_GAIN = 0.05

AL_REVERB_MIN_REFLECTIONS_DELAY = 0.0
AL_REVERB_MAX_REFLECTIONS_DELAY = 0.3
AL_REVERB_DEFAULT_REFLECTIONS_DELAY = 0.007

AL_REVERB_MIN_LATE_REVERB_GAIN = 0.0
AL_REVERB_MAX_LATE_REVERB_GAIN = 10.0
AL_REVERB_DEFAULT_LATE_REVERB_GAIN = 1.26

AL_REVERB_MIN_LATE_REVERB_DELAY = 0.0
AL_REVERB_MAX_LATE_REVERB_DELAY = 0.1
AL_REVERB_DEFAULT_LATE_REVERB_DELAY = 0.011

AL_REVERB_MIN_AIR_ABSORPTION_GAINHF = 0.892
AL_REVERB_MAX_AIR_ABSORPTION_GAINHF = 1.0
AL_REVERB_DEFAULT_AIR_ABSORPTION_GAINHF = 0.994

AL_REVERB_MIN_ROOM_ROLLOFF_FACTOR = 0.0
AL_REVERB_MAX_ROOM_ROLLOFF_FACTOR = 10.0
AL_REVERB_DEFAULT_ROOM_ROLLOFF_FACTOR = 0.0

AL_REVERB_MIN_DECAY_HFLIMIT = AL_FALSE
AL_REVERB_MAX_DECAY_HFLIMIT = AL_TRUE
AL_REVERB_DEFAULT_DECAY_HFLIMIT = AL_TRUE

AL_EAXREVERB_MIN_DENSITY = 0.0
AL_EAXREVERB_MAX_DENSITY = 1.0
AL_EAXREVERB_DEFAULT_DENSITY = 1.0

AL_EAXREVERB_MIN_DIFFUSION = 0.0
AL_EAXREVERB_MAX_DIFFUSION = 1.0
AL_EAXREVERB_DEFAULT_DIFFUSION = 1.0

AL_EAXREVERB_MIN_GAIN = 0.0
AL_EAXREVERB_MAX_GAIN = 1.0
AL_EAXREVERB_DEFAULT_GAIN = 0.32

AL_EAXREVERB_MIN_GAINHF = 0.0
AL_EAXREVERB_MAX_GAINHF = 1.0
AL_EAXREVERB_DEFAULT_GAINHF = 0.89

AL_EAXREVERB_MIN_GAINLF = 0.0
AL_EAXREVERB_MAX_GAINLF = 1.0
AL_EAXREVERB_DEFAULT_GAINLF = 1.0

AL_EAXREVERB_MIN_DECAY_TIME = 0.1
AL_EAXREVERB_MAX_DECAY_TIME = 20.0
AL_EAXREVERB_DEFAULT_DECAY_TIME = 1.49

AL_EAXREVERB_MIN_DECAY_HFRATIO = 0.1
AL_EAXREVERB_MAX_DECAY_HFRATIO = 2.0
AL_EAXREVERB_DEFAULT_DECAY_HFRATIO = 0.83

AL_EAXREVERB_MIN_DECAY_LFRATIO = 0.1
AL_EAXREVERB_MAX_DECAY_LFRATIO = 2.0
AL_EAXREVERB_DEFAULT_DECAY_LFRATIO = 1.0

AL_EAXREVERB_MIN_REFLECTIONS_GAIN = 0.0
AL_EAXREVERB_MAX_REFLECTIONS_GAIN = 3.16
AL_EAXREVERB_DEFAULT_REFLECTIONS_GAIN = 0.05

AL_EAXREVERB_MIN_REFLECTIONS_DELAY = 0.0
AL_EAXREVERB_MAX_REFLECTIONS_DELAY = 0.3
AL_EAXREVERB_DEFAULT_REFLECTIONS_DELAY = 0.007

AL_EAXREVERB_DEFAULT_REFLECTIONS_PAN_XYZ = 0.0

AL_EAXREVERB_MIN_LATE_REVERB_GAIN = 0.0
AL_EAXREVERB_MAX_LATE_REVERB_GAIN = 10.0
AL_EAXREVERB_DEFAULT_LATE_REVERB_GAIN = 1.26

AL_EAXREVERB_MIN_LATE_REVERB_DELAY = 0.0
AL_EAXREVERB_MAX_LATE_REVERB_DELAY = 0.1
AL_EAXREVERB_DEFAULT_LATE_REVERB_DELAY = 0.011

AL_EAXREVERB_DEFAULT_LATE_REVERB_PAN_XYZ = 0.0

AL_EAXREVERB_MIN_ECHO_TIME = 0.075
AL_EAXREVERB_MAX_ECHO_TIME = 0.25
AL_EAXREVERB_DEFAULT_ECHO_TIME = 0.25

AL_EAXREVERB_MIN_ECHO_DEPTH = 0.0
AL_EAXREVERB_MAX_ECHO_DEPTH = 1.0
AL_EAXREVERB_DEFAULT_ECHO_DEPTH = 0.0

AL_EAXREVERB_MIN_MODULATION_TIME = 0.04
AL_EAXREVERB_MAX_MODULATION_TIME = 4.0
AL_EAXREVERB_DEFAULT_MODULATION_TIME = 0.25

AL_EAXREVERB_MIN_MODULATION_DEPTH = 0.0
AL_EAXREVERB_MAX_MODULATION_DEPTH = 1.0
AL_EAXREVERB_DEFAULT_MODULATION_DEPTH = 0.0

AL_EAXREVERB_MIN_AIR_ABSORPTION_GAINHF = 0.892
AL_EAXREVERB_MAX_AIR_ABSORPTION_GAINHF = 1.0
AL_EAXREVERB_DEFAULT_AIR_ABSORPTION_GAINHF = 0.994

AL_EAXREVERB_MIN_HFREFERENCE = 1000.0
AL_EAXREVERB_MAX_HFREFERENCE = 20000.0
AL_EAXREVERB_DEFAULT_HFREFERENCE = 5000.0

AL_EAXREVERB_MIN_LFREFERENCE = 20.0
AL_EAXREVERB_MAX_LFREFERENCE = 1000.0
AL_EAXREVERB_DEFAULT_LFREFERENCE = 250.0

AL_EAXREVERB_MIN_ROOM_ROLLOFF_FACTOR = 0.0
AL_EAXREVERB_MAX_ROOM_ROLLOFF_FACTOR = 10.0
AL_EAXREVERB_DEFAULT_ROOM_ROLLOFF_FACTOR = 0.0

AL_EAXREVERB_MIN_DECAY_HFLIMIT = AL_FALSE
AL_EAXREVERB_MAX_DECAY_HFLIMIT = AL_TRUE
AL_EAXREVERB_DEFAULT_DECAY_HFLIMIT = AL_TRUE

AL_CHORUS_WAVEFORM_SINUSOID = 0
AL_CHORUS_WAVEFORM_TRIANGLE = 1

AL_CHORUS_MIN_WAVEFORM = 0
AL_CHORUS_MAX_WAVEFORM = 1
AL_CHORUS_DEFAULT_WAVEFORM = 1

AL_CHORUS_MIN_PHASE = -180
AL_CHORUS_MAX_PHASE = 180
AL_CHORUS_DEFAULT_PHASE = 90

AL_CHORUS_MIN_RATE = 0.0
AL_CHORUS_MAX_RATE = 10.0
AL_CHORUS_DEFAULT_RATE = 1.1

AL_CHORUS_MIN_DEPTH = 0.0
AL_CHORUS_MAX_DEPTH = 1.0
AL_CHORUS_DEFAULT_DEPTH = 0.1

AL_CHORUS_MIN_FEEDBACK = -1.0
AL_CHORUS_MAX_FEEDBACK = 1.0
AL_CHORUS_DEFAULT_FEEDBACK = 0.25

AL_CHORUS_MIN_DELAY = 0.0
AL_CHORUS_MAX_DELAY = 0.016
AL_CHORUS_DEFAULT_DELAY = 0.016

AL_DISTORTION_MIN_EDGE = 0.0
AL_DISTORTION_MAX_EDGE = 1.0
AL_DISTORTION_DEFAULT_EDGE = 0.2

AL_DISTORTION_MIN_GAIN = 0.01
AL_DISTORTION_MAX_GAIN = 1.0
AL_DISTORTION_DEFAULT_GAIN = 0.05

AL_DISTORTION_MIN_LOWPASS_CUTOFF = 80.0
AL_DISTORTION_MAX_LOWPASS_CUTOFF = 24000.0
AL_DISTORTION_DEFAULT_LOWPASS_CUTOFF = 8000.0

AL_DISTORTION_MIN_EQCENTER = 80.0
AL_DISTORTION_MAX_EQCENTER = 24000.0
AL_DISTORTION_DEFAULT_EQCENTER = 3600.0

AL_DISTORTION_MIN_EQBANDWIDTH = 80.0
AL_DISTORTION_MAX_EQBANDWIDTH = 24000.0
AL_DISTORTION_DEFAULT_EQBANDWIDTH = 3600.0

AL_ECHO_MIN_DELAY = 0.0
AL_ECHO_MAX_DELAY = 0.207
AL_ECHO_DEFAULT_DELAY = 0.1

AL_ECHO_MIN_LRDELAY = 0.0
AL_ECHO_MAX_LRDELAY = 0.404
AL_ECHO_DEFAULT_LRDELAY = 0.1

AL_ECHO_MIN_DAMPING = 0.0
AL_ECHO_MAX_DAMPING = 0.99
AL_ECHO_DEFAULT_DAMPING = 0.5

AL_ECHO_MIN_FEEDBACK = 0.0
AL_ECHO_MAX_FEEDBACK = 1.0
AL_ECHO_DEFAULT_FEEDBACK = 0.5

AL_ECHO_MIN_SPREAD = -1.0
AL_ECHO_MAX_SPREAD = 1.0
AL_ECHO_DEFAULT_SPREAD = -1.0

AL_FLANGER_WAVEFORM_SINUSOID = 0
AL_FLANGER_WAVEFORM_TRIANGLE = 1

AL_FLANGER_MIN_WAVEFORM = 0
AL_FLANGER_MAX_WAVEFORM = 1
AL_FLANGER_DEFAULT_WAVEFORM = 1

AL_FLANGER_MIN_PHASE = -180
AL_FLANGER_MAX_PHASE = 180
AL_FLANGER_DEFAULT_PHASE = 0

AL_FLANGER_MIN_RATE = 0.0
AL_FLANGER_MAX_RATE = 10.0
AL_FLANGER_DEFAULT_RATE = 0.27

AL_FLANGER_MIN_DEPTH = 0.0
AL_FLANGER_MAX_DEPTH = 1.0
AL_FLANGER_DEFAULT_DEPTH = 1.0

AL_FLANGER_MIN_FEEDBACK = -1.0
AL_FLANGER_MAX_FEEDBACK = 1.0
AL_FLANGER_DEFAULT_FEEDBACK = -0.5

AL_FLANGER_MIN_DELAY = 0.0
AL_FLANGER_MAX_DELAY = 0.004
AL_FLANGER_DEFAULT_DELAY = 0.002

AL_FREQUENCY_SHIFTER_MIN_FREQUENCY = 0.0
AL_FREQUENCY_SHIFTER_MAX_FREQUENCY = 24000.0
AL_FREQUENCY_SHIFTER_DEFAULT_FREQUENCY = 0.0

AL_FREQUENCY_SHIFTER_MIN_LEFT_DIRECTION = 0
AL_FREQUENCY_SHIFTER_MAX_LEFT_DIRECTION = 2
AL_FREQUENCY_SHIFTER_DEFAULT_LEFT_DIRECTION = 0

AL_FREQUENCY_SHIFTER_DIRECTION_DOWN = 0
AL_FREQUENCY_SHIFTER_DIRECTION_UP = 1
AL_FREQUENCY_SHIFTER_DIRECTION_OFF = 2

AL_FREQUENCY_SHIFTER_MIN_RIGHT_DIRECTION = 0
AL_FREQUENCY_SHIFTER_MAX_RIGHT_DIRECTION = 2
AL_FREQUENCY_SHIFTER_DEFAULT_RIGHT_DIRECTION = 0

AL_VOCAL_MORPHER_MIN_PHONEMEA = 0
AL_VOCAL_MORPHER_MAX_PHONEMEA = 29
AL_VOCAL_MORPHER_DEFAULT_PHONEMEA = 0

AL_VOCAL_MORPHER_MIN_PHONEMEA_COARSE_TUNING = -24
AL_VOCAL_MORPHER_MAX_PHONEMEA_COARSE_TUNING = 24
AL_VOCAL_MORPHER_DEFAULT_PHONEMEA_COARSE_TUNING = 0

AL_VOCAL_MORPHER_MIN_PHONEMEB = 0
AL_VOCAL_MORPHER_MAX_PHONEMEB = 29
AL_VOCAL_MORPHER_DEFAULT_PHONEMEB = 10

AL_VOCAL_MORPHER_MIN_PHONEMEB_COARSE_TUNING = -24
AL_VOCAL_MORPHER_MAX_PHONEMEB_COARSE_TUNING = 24
AL_VOCAL_MORPHER_DEFAULT_PHONEMEB_COARSE_TUNING = 0

AL_VOCAL_MORPHER_PHONEME_A = 0
AL_VOCAL_MORPHER_PHONEME_E = 1
AL_VOCAL_MORPHER_PHONEME_I = 2
AL_VOCAL_MORPHER_PHONEME_O = 3
AL_VOCAL_MORPHER_PHONEME_U = 4
AL_VOCAL_MORPHER_PHONEME_AA = 5
AL_VOCAL_MORPHER_PHONEME_AE = 6
AL_VOCAL_MORPHER_PHONEME_AH = 7
AL_VOCAL_MORPHER_PHONEME_AO = 8
AL_VOCAL_MORPHER_PHONEME_EH = 9
AL_VOCAL_MORPHER_PHONEME_ER = 10
AL_VOCAL_MORPHER_PHONEME_IH = 11
AL_VOCAL_MORPHER_PHONEME_IY = 12
AL_VOCAL_MORPHER_PHONEME_UH = 13
AL_VOCAL_MORPHER_PHONEME_UW = 14
AL_VOCAL_MORPHER_PHONEME_B = 15
AL_VOCAL_MORPHER_PHONEME_D = 16
AL_VOCAL_MORPHER_PHONEME_F = 17
AL_VOCAL_MORPHER_PHONEME_G = 18
AL_VOCAL_MORPHER_PHONEME_J = 19
AL_VOCAL_MORPHER_PHONEME_K = 20
AL_VOCAL_MORPHER_PHONEME_L = 21
AL_VOCAL_MORPHER_PHONEME_M = 22
AL_VOCAL_MORPHER_PHONEME_N = 23
AL_VOCAL_MORPHER_PHONEME_P = 24
AL_VOCAL_MORPHER_PHONEME_R = 25
AL_VOCAL_MORPHER_PHONEME_S = 26
AL_VOCAL_MORPHER_PHONEME_T = 27
AL_VOCAL_MORPHER_PHONEME_V = 28
AL_VOCAL_MORPHER_PHONEME_Z = 29

AL_VOCAL_MORPHER_WAVEFORM_SINUSOID = 0
AL_VOCAL_MORPHER_WAVEFORM_TRIANGLE = 1
AL_VOCAL_MORPHER_WAVEFORM_SAWTOOTH = 2

AL_VOCAL_MORPHER_MIN_WAVEFORM = 0
AL_VOCAL_MORPHER_MAX_WAVEFORM = 2
AL_VOCAL_MORPHER_DEFAULT_WAVEFORM = 0

AL_VOCAL_MORPHER_MIN_RATE = 0.0
AL_VOCAL_MORPHER_MAX_RATE = 10.0
AL_VOCAL_MORPHER_DEFAULT_RATE = 1.41

AL_PITCH_SHIFTER_MIN_COARSE_TUNE = -12
AL_PITCH_SHIFTER_MAX_COARSE_TUNE = 12
AL_PITCH_SHIFTER_DEFAULT_COARSE_TUNE = 12

AL_PITCH_SHIFTER_MIN_FINE_TUNE = -50
AL_PITCH_SHIFTER_MAX_FINE_TUNE = 50
AL_PITCH_SHIFTER_DEFAULT_FINE_TUNE = 0

AL_RING_MODULATOR_MIN_FREQUENCY = 0.0
AL_RING_MODULATOR_MAX_FREQUENCY = 8000.0
AL_RING_MODULATOR_DEFAULT_FREQUENCY = 440.0

AL_RING_MODULATOR_MIN_HIGHPASS_CUTOFF = 0.0
AL_RING_MODULATOR_MAX_HIGHPASS_CUTOFF = 24000.0
AL_RING_MODULATOR_DEFAULT_HIGHPASS_CUTOFF = 800.0

AL_RING_MODULATOR_SINUSOID = 0
AL_RING_MODULATOR_SAWTOOTH = 1
AL_RING_MODULATOR_SQUARE = 2

AL_RING_MODULATOR_MIN_WAVEFORM = 0
AL_RING_MODULATOR_MAX_WAVEFORM = 2
AL_RING_MODULATOR_DEFAULT_WAVEFORM = 0

AL_AUTOWAH_MIN_ATTACK_TIME = 0.0001
AL_AUTOWAH_MAX_ATTACK_TIME = 1.0
AL_AUTOWAH_DEFAULT_ATTACK_TIME = 0.06

AL_AUTOWAH_MIN_RELEASE_TIME = 0.0001
AL_AUTOWAH_MAX_RELEASE_TIME = 1.0
AL_AUTOWAH_DEFAULT_RELEASE_TIME = 0.06

AL_AUTOWAH_MIN_RESONANCE = 2.0
AL_AUTOWAH_MAX_RESONANCE = 1000.0
AL_AUTOWAH_DEFAULT_RESONANCE = 1000.0

AL_AUTOWAH_MIN_PEAK_GAIN = 0.00003
AL_AUTOWAH_MAX_PEAK_GAIN = 31621.0
AL_AUTOWAH_DEFAULT_PEAK_GAIN = 11.22

AL_COMPRESSOR_MIN_ONOFF = 0
AL_COMPRESSOR_MAX_ONOFF = 1
AL_COMPRESSOR_DEFAULT_ONOFF = 1

AL_EQUALIZER_MIN_LOW_GAIN = 0.126
AL_EQUALIZER_MAX_LOW_GAIN = 7.943
AL_EQUALIZER_DEFAULT_LOW_GAIN = 1.0

AL_EQUALIZER_MIN_LOW_CUTOFF = 50.0
AL_EQUALIZER_MAX_LOW_CUTOFF = 800.0
AL_EQUALIZER_DEFAULT_LOW_CUTOFF = 200.0

AL_EQUALIZER_MIN_MID1_GAIN = 0.126
AL_EQUALIZER_MAX_MID1_GAIN = 7.943
AL_EQUALIZER_DEFAULT_MID1_GAIN = 1.0

AL_EQUALIZER_MIN_MID1_CENTER = 200.0
AL_EQUALIZER_MAX_MID1_CENTER = 3000.0
AL_EQUALIZER_DEFAULT_MID1_CENTER = 500.0

AL_EQUALIZER_MIN_MID1_WIDTH = 0.01
AL_EQUALIZER_MAX_MID1_WIDTH = 1.0
AL_EQUALIZER_DEFAULT_MID1_WIDTH = 1.0

AL_EQUALIZER_MIN_MID2_GAIN = 0.126
AL_EQUALIZER_MAX_MID2_GAIN = 7.943
AL_EQUALIZER_DEFAULT_MID2_GAIN = 1.0

AL_EQUALIZER_MIN_MID2_CENTER = 1000.0
AL_EQUALIZER_MAX_MID2_CENTER = 8000.0
AL_EQUALIZER_DEFAULT_MID2_CENTER = 3000.0

AL_EQUALIZER_MIN_MID2_WIDTH = 0.01
AL_EQUALIZER_MAX_MID2_WIDTH = 1.0
AL_EQUALIZER_DEFAULT_MID2_WIDTH = 1.0

AL_EQUALIZER_MIN_HIGH_GAIN = 0.126
AL_EQUALIZER_MAX_HIGH_GAIN = 7.943
AL_EQUALIZER_DEFAULT_HIGH_GAIN = 1.0

AL_EQUALIZER_MIN_HIGH_CUTOFF = 4000.0
AL_EQUALIZER_MAX_HIGH_CUTOFF = 16000.0
AL_EQUALIZER_DEFAULT_HIGH_CUTOFF = 6000.0

AL_MIN_AIR_ABSORPTION_FACTOR = 0.0
AL_MAX_AIR_ABSORPTION_FACTOR = 10.0
AL_DEFAULT_AIR_ABSORPTION_FACTOR = 0.0

AL_MIN_ROOM_ROLLOFF_FACTOR = 0.0
AL_MAX_ROOM_ROLLOFF_FACTOR = 10.0
AL_DEFAULT_ROOM_ROLLOFF_FACTOR = 0.0

AL_MIN_CONE_OUTER_GAINHF = 0.0
AL_MAX_CONE_OUTER_GAINHF = 1.0
AL_DEFAULT_CONE_OUTER_GAINHF = 1.0

AL_MIN_DIRECT_FILTER_GAINHF_AUTO = AL_FALSE
AL_MAX_DIRECT_FILTER_GAINHF_AUTO = AL_TRUE
AL_DEFAULT_DIRECT_FILTER_GAINHF_AUTO = AL_TRUE

AL_MIN_AUXILIARY_SEND_FILTER_GAIN_AUTO = AL_FALSE
AL_MAX_AUXILIARY_SEND_FILTER_GAIN_AUTO = AL_TRUE
AL_DEFAULT_AUXILIARY_SEND_FILTER_GAIN_AUTO = AL_TRUE

AL_MIN_AUXILIARY_SEND_FILTER_GAINHF_AUTO = AL_FALSE
AL_MAX_AUXILIARY_SEND_FILTER_GAINHF_AUTO = AL_TRUE
AL_DEFAULT_AUXILIARY_SEND_FILTER_GAINHF_AUTO = AL_TRUE

AL_MIN_METERS_PER_UNIT = 1e-37
AL_MAX_METERS_PER_UNIT = 1e+37
AL_DEFAULT_METERS_PER_UNIT = 1.0

alGenEffects = _lib.alGenEffects
alGenEffects.restype = None
alGenEffects.argtypes = [ALsizei, POINTER(ALuint)]

alDeleteEffects = _lib.alDeleteEffects
alDeleteEffects.restype = None
alDeleteEffects.argtypes = [ALsizei, POINTER(ALuint)]

alIsEffect = _lib.alIsEffect
alIsEffect.restype = ALboolean
alIsEffect.argtypes = [ALuint]

alEffecti = _lib.alEffecti
alEffecti.restype = None
alEffecti.argtypes = [ALuint, ALenum, ALint]

alEffectiv = _lib.alEffectiv
alEffectiv.restype = None
alEffectiv.argtypes = [ALuint, ALenum, POINTER(ALint)]

alEffectf = _lib.alEffectf
alEffectf.restype = None
alEffectf.argtypes = [ALuint, ALenum, ALfloat]

alEffectfv = _lib.alEffectfv
alEffectfv.restype = None
alEffectfv.argtypes = [ALuint, ALenum, POINTER(ALfloat)]

alGetEffecti = _lib.alGetEffecti
alGetEffecti.restype = None
alGetEffecti.argtypes = [ALuint, ALenum, POINTER(ALint)]

alGetEffectiv = _lib.alGetEffectiv
alGetEffectiv.restype = None
alGetEffectiv.argtypes = [ALuint, ALenum, POINTER(ALint)]

alGetEffectf = _lib.alGetEffectf
alGetEffectf.restype = None
alGetEffectf.argtypes = [ALuint, ALenum, POINTER(ALfloat)]

alGetEffectfv = _lib.alGetEffectfv
alGetEffectfv.restype = None
alGetEffectfv.argtypes = [ALuint, ALenum, POINTER(ALfloat)]

alGenFilters = _lib.alGenFilters
alGenFilters.restype = None
alGenFilters.argtypes = [ALsizei, POINTER(ALuint)]

alDeleteFilters = _lib.alDeleteFilters
alDeleteFilters.restype = None
alDeleteFilters.argtypes = [ALsizei, POINTER(ALuint)]

alIsFilter = _lib.alIsFilter
alIsFilter.restype = ALboolean
alIsFilter.argtypes = [ALuint]

alFilteri = _lib.alFilteri
alFilteri.restype = None
alFilteri.argtypes = [ALuint, ALenum, ALint]

alFilteriv = _lib.alFilteriv
alFilteriv.restype = None
alFilteriv.argtypes = [ALuint, ALenum, POINTER(ALint)]

alFilterf = _lib.alFilterf
alFilterf.restype = None
alFilterf.argtypes = [ALuint, ALenum, ALfloat]

alFilterfv = _lib.alFilterfv
alFilterfv.restype = None
alFilterfv.argtypes = [ALuint, ALenum, POINTER(ALfloat)]

alGetFilteri = _lib.alGetFilteri
alGetFilteri.restype = None
alGetFilteri.argtypes = [ALuint, ALenum, POINTER(ALint)]

alGetFilteriv = _lib.alGetFilteriv
alGetFilteriv.restype = None
alGetFilteriv.argtypes = [ALuint, ALenum, POINTER(ALint)]

alGetFilterf = _lib.alGetFilterf
alGetFilterf.restype = None
alGetFilterf.argtypes = [ALuint, ALenum, POINTER(ALfloat)]

alGetFilterfv = _lib.alGetFilterfv
alGetFilterfv.restype = None
alGetFilterfv.argtypes = [ALuint, ALenum, POINTER(ALfloat)]

alGenAuxiliaryEffectSlots = _lib.alGenAuxiliaryEffectSlots
alGenAuxiliaryEffectSlots.restype = None
alGenAuxiliaryEffectSlots.argtypes = [ALsizei, POINTER(ALuint)]

alDeleteAuxiliaryEffectSlots = _lib.alDeleteAuxiliaryEffectSlots
alDeleteAuxiliaryEffectSlots.restype = None
alDeleteAuxiliaryEffectSlots.argtype = [ALsizei, POINTER(ALuint)]

alIsAuxiliaryEffectSlot = _lib.alIsAuxiliaryEffectSlot
alIsAuxiliaryEffectSlot.restype = ALboolean
alIsAuxiliaryEffectSlot.argtypes = [ALuint]

alAuxiliaryEffectSloti = _lib.alAuxiliaryEffectSloti
alAuxiliaryEffectSloti.restype = None
alAuxiliaryEffectSloti.argtypes = [ALuint, ALenum, ALint]

alAuxiliaryEffectSlotiv = _lib.alAuxiliaryEffectSlotiv
alAuxiliaryEffectSlotiv.restype = None
alAuxiliaryEffectSlotiv.argtypes = [ALuint, ALenum, POINTER(ALint)]

alAuxiliaryEffectSlotf = _lib.alAuxiliaryEffectSlotf
alAuxiliaryEffectSlotf.restype = None
alAuxiliaryEffectSlotf.argtypes = [ALuint, ALenum, ALfloat]

alAuxiliaryEffectSlotfv = _lib.alAuxiliaryEffectSlotfv
alAuxiliaryEffectSlotfv.restype = None
alAuxiliaryEffectSlotfv.argtypes = [ALuint, ALenum, POINTER(ALfloat)]

alGetAuxiliaryEffectSloti = _lib.alGetAuxiliaryEffectSloti
alGetAuxiliaryEffectSloti.restype = None
alGetAuxiliaryEffectSloti.argtypes = [ALuint, ALenum, POINTER(ALint)]

alGetAuxiliaryEffectSlotiv = _lib.alGetAuxiliaryEffectSlotiv
alGetAuxiliaryEffectSlotiv.restype = None
alGetAuxiliaryEffectSlotiv.argtypes = [ALuint, ALenum, POINTER(ALint)]

alGetAuxiliaryEffectSlotf = _lib.alGetAuxiliaryEffectSlotf
alGetAuxiliaryEffectSlotf.restype = None
alGetAuxiliaryEffectSlotf.argtypes = [ALuint, ALenum, POINTER(ALfloat)]

alGetAuxiliaryEffectSlotfv = _lib.alGetAuxiliaryEffectSlotfv
alGetAuxiliaryEffectSlotfv.restype = None
alGetAuxiliaryEffectSlotfv.argtypes = [ALuint, ALenum, POINTER(ALfloat)]

LPALGENEFFECTS = CFUNCTYPE(None, ALsizei, POINTER(ALuint))
LPALDELETEEFECTS = CFUNCTYPE(None, ALsizei, POINTER(ALuint))
LPALISEFFECT = CFUNCTYPE(ALboolean, ALuint)
LPALEFFECTI = CFUNCTYPE(None, ALuint, ALenum, ALint)
LPALEFFECTIV = CFUNCTYPE(None, ALuint, ALenum, POINTER(ALint))
LPALEFFECTF = CFUNCTYPE(None, ALuint, ALenum, ALfloat)
LPALEFFECTFV = CFUNCTYPE(None, ALuint, ALenum, POINTER(ALfloat))
LPALGETEFFECTI = CFUNCTYPE(None, ALuint, ALenum, POINTER(ALint))
LPALGETEFFECTIV = CFUNCTYPE(None, ALuint, ALenum, POINTER(ALint))
LPALGETEFFECTF = CFUNCTYPE(None, ALuint, ALenum, POINTER(ALfloat))
LPALGETEFFECTFV = CFUNCTYPE(None, ALuint, ALenum, POINTER(ALfloat))
LPALGENFILTERS = CFUNCTYPE(None, ALsizei, POINTER(ALuint))
LPALDELETEFILTERS = CFUNCTYPE(None, ALsizei, POINTER(ALuint))
LPALISFILTER = CFUNCTYPE(ALboolean, ALuint)
LPALFILTERI = CFUNCTYPE(None, ALuint, ALenum, ALint)
LPALFILTERIV = CFUNCTYPE(None, ALuint, ALenum, POINTER(ALint))
LPALFILTERF = CFUNCTYPE(None, ALuint, ALenum, ALfloat)
LPALFILTERFV = CFUNCTYPE(None, ALuint, ALenum, POINTER(ALfloat))
LPALGETFILTERI = CFUNCTYPE(None, ALuint, ALenum, POINTER(ALint))
LPALGETFILTERIV = CFUNCTYPE(None, ALuint, ALenum, POINTER(ALint))
LPALGETFILTERF = CFUNCTYPE(None, ALuint, ALenum, POINTER(ALfloat))
LPALGETFILTERFV = CFUNCTYPE(None, ALuint, ALenum, POINTER(ALfloat))
ALGENAUXILIARYEFFECTSLOTS = CFUNCTYPE(None, ALsizei, POINTER(ALuint))
ALDELETEAUXILIARYEFFECTSLOTS = CFUNCTYPE(None, ALsizei, POINTER(ALuint))
ALISAUXILIARYEFFECTSLOT = CFUNCTYPE(ALboolean, ALuint)
ALAUXILIARYEFFECTSLOTI = CFUNCTYPE(None, ALuint, ALenum, ALint)
ALAUXILIARYEFFECTSLOTIV = CFUNCTYPE(None, ALuint, ALenum, POINTER(ALint))
ALAUXILIARYEFFECTSLOTF = CFUNCTYPE(None, ALuint, ALenum, ALfloat)
ALAUXILIARYEFFECTSLOTFV = CFUNCTYPE(None, ALuint, ALenum, POINTER(ALfloat))
ALGETAUXILIARYEFFECTSLOTI = CFUNCTYPE(None, ALuint, ALenum, POINTER(ALint))
ALGETAUXILIARYEFFECTSLOTIV = CFUNCTYPE(None, ALuint, ALenum, POINTER(ALint))
ALGETAUXILIARYEFFECTSLOTF = CFUNCTYPE(None, ALuint, ALenum, POINTER(ALfloat))
ALGETAUXILIARYEFFECTSLOTFV = CFUNCTYPE(None, ALuint, ALenum, POINTER(ALfloat))

__all__ = ['ALsizei', 'ALfloat', 'ALenum', 'ALuint', 'ALint', 'ALboolean',
'AL_TRUE', 'AL_FALSE', 'ALC_EXT_EFX_NAME', 'ALC_EFX_MAJOR_VERSION',
'ALC_EFX_MINOR_VERSION', 'ALC_MAX_AUXILIARY_SENDS',  'AL_METERS_PER_UNIT',
'AL_DIRECT_FILTER', 'AL_AUXILIARY_SEND_FILTER', 'AL_AIR_ABSORPTION_FACTOR',
'AL_ROOM_ROLLOFF_FACTOR', 'AL_CONE_OUTER_GAINHF',
'AL_DIRECT_FILTER_GAINHF_AUTO', 'AL_AUXILIARY_SEND_FILTER_GAIN_AUTO',
'AL_AUXILIARYSEND_FILTER_GAINHF_AUTO', 'AL_REVERB_DENSITY',
'AL_REVERB_DIFFUSION', 'AL_REVERB_GAIN', 'AL_REVERB_GAINHF',
'AL_REVERB_DECAY_TIME', 'AL_REVERB_DECAY_HFRATIO', 'AL_REVERB_REFLECTIONS_GAIN',
'AL_REVERB_REFLECTIONS_DELAY', 'AL_REVERB_LATE_REVERB_GAIN',
'AL_REVERB_LATE_REVERB_DELAY', 'AL_REVERB_AIR_ABSORPTION_GAINHF',
'AL_REVERB_ROOM_ROLLOFF_FACTOR', 'AL_REVERB_DECAY_HFLIMIT',
'AL_EAXREVERB_DENSITY', 'AL_EAXREVERB_DIFFUSION', 'AL_EAXREVERB_GAIN',
'AL_EAXREVERB_GAINHF', 'AL_EAXREVERB_GAINLF', 'AL_EAXREVERB_DECAY_TIME',
'AL_EAXREVERB_DECAY_HFRATIO', 'AL_EAXREVERB_LFRATIO',
'AL_EAXREVERB_REFLECTIONS_GAIN', 'AL_EAXREVERB_REFLECTIONS DELAY',
'AL_EAXREVERB_REFLECTIONS_PAN', 'AL_EAXREVERB_LATE_REVERB_GAIN',
'AL_EAXREVERB_LATE_REVERB_DELAY', 'AL_EAXREVERB_LATE_REVERB_PAN',
'AL_EAXREVERB_ECHO_TIME', 'AL_EAXREVERB_ECHO_DEPTH',
'AL_EAXREVERB_MODULATION_TIME', 'AL_EAXREVERB_MODULATION_DEPTH',
'AL_EAXREVERB_AIR_ABSORPTION_GAINHF', 'AL_EAXREVERB_HFREFERENCE',
'AL_EAXREVERB_LFREFERENCE', 'AL_EAXREVERB_ROOM_ROLLOFF_FACTOR',
'AL_EAXREVERB_DECAY_HFLIMIT', 'AL_CHORUS_WAVEFORM', 'AL_CHORUS_PHASE',
'AL_CHORUS_RATE', 'AL_CHORUS_DEPTH', 'AL_CHORUS_FEEDBACK', 'AL_CHORUS_DELAY',
'AL_DISTORTION_EDGE', 'AL_DISTORTION_GAIN', 'AL_DISTORTION_LOWPASS_CUTOFF',
'AL_DISTORTION_EQCENTER', 'AL_DISTORTION_EQBANDWIDTH', 'AL_ECHO_DELAY',
'AL_ECHO_LRDELAY', 'AL_ECHO_DAMPING', 'AL_ECHO_FEEDBACK', 'AL_ECHO_SPREAD',
'AL_FLANGER_WAVEFORM', 'AL_FLANGER_PHASE', 'AL_FLANGER_RATE',
'AL_FLANGER_DEPTH', 'AL_FLANGER_FEEDBACK', 'AL_FLANGER_DELAY',
'AL_FREQUENCY_SHIFTER_FREQUENCY', 'AL_FREQUENCY_SHIFTER_LEFT_DIRECTION',
'AL_FREQUENCY_SHIFTER_RIGHT_DIRECTION', 'AL_VOCAL_MORPHER_PHONEMEA',
'AL_VOCAL_MORPHER_PHONEMEA_COARSE_TUNING', 'AL_VOCAL_MORPHER_PHONEMEB',
'AL_VOCAL_MORPHER_PHONEMEB_COARSE_TUNING', 'AL_VOCAL_MORPHER_WAVEFORM',
'AL_VOCAL_MORPHER_RATE', 'AL_PITCH_SHIFTER_COARSE_TUNE',
'AL_PITCH_SHIFTER_FINE_TUNE', 'AL_RING_MODULATOR_FREQUENCY',
'AL_RING_MODULATOR_HIGHPASS_CUTOFF', 'AL_RING_MODULATOR_WAVEFORM',
'AL_AUTOWAH_ATTACK_TIME', 'AL_AUTOWAH_RELEASE_TIME', 'AL_AUTOWAH_RESONANCE',
'AL_AUTOWAH_PEAK_GAIN', 'AL_COMPRESSOR_ONOFF', 'AL_EQUALIZER_LOW_GAIN',
'AL_EQUALIZER_LOW_CUTOFF', 'AL_EQUALIZER_MID1_GAIN', 'AL_EQUALIZER_MID1_CENTER',
'AL_EQUALIZER_MID1_WIDTH', 'AL_EQUALIZER_MID2_GAIN', 'AL_EQUALIZER_MID2_CENTER',
'AL_EQUALIZER_MID2_WIDTH', 'AL_EQUALIZER_HIGH_GAIN', 'AL_EQUALIZER_HIGH_CUTOFF',
'AL_EFFECT_FIRST_PARAMETER', 'AL_EFFECT_LAST_PARAMETER', 'AL_EFFECT_TYPE',
'AL_EFFECT_NULL', 'AL_EFFECT_REVERB', 'AL_EFFECT_CHORUS',
'AL_EFFECT_DISTORTION', 'AL_EFFECT_ECHO', 'AL_EFFECT_FLANGER',
'AL_EFFECT_FREQUENCY_SHIFTER', 'AL_EFFECT_VOCAL_MORPHER',
'AL_EFFECT_PITCH_SHIFTER', 'AL_EFFECT_RING_MODULATOR', 'AL_EFFECT_AUTOWAH',
'AL_EFFECT_COMPRESSOR', 'AL_EFFECT_EQUALIZER', 'AL_EFFECT_EAXREVERB',
'AL_EFFECTSLOT_EFFECT', 'AL_EFFECTSLOT_GAIN',
'AL_EFFECTSLOT_AUXILIARY_SEND_AUTO', 'AL_EFFECTSLOT_NULL', 'AL_LOWPASS_GAIN',
'AL_LOWPASS_GAINHF', 'AL_BANDPASS_GAIN', 'AL_BANDPASS_GAINLF',
'AL_BANDPASS_GAINHF', 'AL_FILTER_FIRST_PARAMETER', 'AL_FILTER_LAST_PARAMETER',
'AL_FILTER_TYPE', 'AL_FILTER_NULL', 'AL_FILTER_LOWPASS', 'AL_FILTER_HIGHPASS',
'AL_FILTER_BANDPASS', 'AL_LOWPASS_MIN_GAIN', 'AL_LOWPASS_MAX_GAIN',
'AL_LOWPASS_DEFAULT_GAIN', 'AL_LOWPASS_MIN_GAINHF', 'AL_LOWPASS_MAX_GAINHF',
'AL_LOWPASS_DEFAULT_GAINHF', 'AL_HIGHPASS_MIN_GAIN', 'AL_HIGHPASS_MAX_GAIN',
'AL_HIGHPASS_DEFAULT_GAIN', 'AL_HIGHPASS_MIN_GAINLF', 'AL_HIGHPASS_MAX_GAINLF',
'AL_HIGHPASS_DEFAULT_GAINLF', 'AL_BANDPASS_MIN_GAIN', 'AL_BANDPASS_MAX_GAIN',
'AL_BANDPASS_DEFAULT_GAIN', 'AL_BANDPASS_MIN_GAINHF', 'AL_BANDPASS_MAX_GAINHF',
'AL_BANDPASS_DEFAULT_GAINHF', 'AL_BANDPASS_MIN_GAINLF',
'AL_BANDPASS_MAX_GAINLF', 'AL_BANDPASS_DEFAULT_GAINLF',
'AL_REVERB_MIN_DENSITY', 'AL_REVERB_MAX_DENSITY', 'AL_REVERB_DEFAULT_DENSITY',
'AL_REVERB_MIN_DIFFUSION', 'AL_REVERB_MAX_DIFFUSION',
'AL_REVERB_DEFAULT_DIFFUSION', 'AL_REVERB_MIN_GAIN', 'AL_REVERB_MAX GAIN',
'AL_REVERB_DEFAULT_GAIN', 'AL_REVERB_MIN_GAINHF', 'AL_REVERB_MAX_GAINHF',
'AL_REVERB_DEFAULT_GAINHF', 'AL_REVERB_MIN_DECAY_TIME',
'AL_REVERB_MAX_DECAY_TIME', 'AL_REVERB_DEFAULT_DECAY_TIME',
'AL_REVERB_MIN_DECAY_RATIO', 'AL_REVERB_MAX_DECAY_RATIO',
'AL_REVERB_DEFAULT_DECAY_RATIO', 'AL_REVERB_MIN_REFLECTIONS_GAIN',
'AL_REVERB_MAX_REFLECTIONS_GAIN', 'AL_REVERB_DEFAULT_REFLECTIONS_GAIN',
'AL_REVERB_MIN_REFLECTIONS_DELAY', 'AL_REVERB_MAX_REFLECTIONS_DELAY',
'AL_REVERB_DEFAULT_REFLECTIONS_DELAY', 'AL_REVERB_MIN_LATE_REVERB_GAIN',
'AL_REVERB_MAX_LATE_REVERB_GAIN', 'AL_REVERB_DEFAULT_LATE_REVERB_GAIN',
'AL_REVERB_MIN_LATE_REVERB_DELAY', 'AL_REVERB_MAX_LATE_REVERB_DELAY',
'AL_REVERB_DEFAULT_LATE_REVERB_DELAY', 'AL_REVERB_MIN_AIR_ABSORPTION_GAINHF',
'AL_REVERB_MAX_AIR_ABSORPTION_GAINHF', 'AL_REVERB_DEFAULT_AIR_ABSORPTION_GAINHF',
'AL_REVERB_MIN_ROOM_ROLLOFF_FACTOR', 'AL_REVERB_MAX_ROOM_ROLLOFF_FACTOR',
'AL_REVERB_DEFAULT_ROOM_ROLLOFF_FACTOR', 'AL_REVERB_MIN_DECAY_HFLIMIT',
'AL_REVERB_MAX_DECAY_HFLIMIT', 'AL_REVERB_DEFAULT_DECAY_HFLIMIT',
'AL_EAXREVERB_MIN_DENSITY', 'AL_EAXREVERB_MAX_DENSITY',
'AL_EAXREVERB_DEFAULT_DENSITY', 'AL_EAXREVERB_MIN_DIFFUSION',
'AL_EAXREVERB_MAX_DIFFUSION', 'AL_EAXREVERB_DEFAULT_DIFFUSION',
'AL_EAXREVERB_MIN_GAIN', 'AL_EAXREVERB_MAX_GAIN', 'AL_EAXREVERB_DEFAULT_GAIN',
'AL_EAXREVERB_MIN_GAINHF', 'AL_EAXREVERB_MAX_GAINHF',
'AL_EAXREVERB_DEFAULT_GAINHF', 'AL_EAXREVERB_MIN_GAINLF',
'AL_EAXREVERB_MAX_GAINLF', 'AL_EAXREVERB_DEFAULT_GAINLF',
'AL_EAXREVERB_MIN_DECAY_TIME', 'AL_EAXREVERB_MAX_DECAY_TIME',
'AL_EAXREVERB_DEFAULT_DECAY_TIME', 'AL_EAXREVERB_MIN_DECAY_HFRATIO',
'AL_EAXREVERB_MAX_DECAY_HFRATIO', 'AL_EAXREVERB_DEFAULT_DECAY_HFRATIO',
'AL_EAXREVERB_MIN_DECAY_LFRATIO', 'AL_EAXREVERB_MAX_DECAY_LFRATIO',
'AL_EAXREVERB_DEFAULT_DECAY_LFRATIO', 'AL_EAXREVERB_MIN_REFLECTIONS_GAIN',
'AL_EAXREVERB_MAX_REFLECTIONS_GAIN', 'AL_EAXREVERB_DEFAULT_REFLECTIONS_GAIN',
'AL_EAXREVERB_MIN_REFLECTIONS_DELAY', 'AL_EAXREVERB_MAX_REFLECTIONS_DELAY',
'AL_EAXREVERB_DEFAULT_REFLECTIONS_DELAY',
'AL_EAXREVERB_DEFAULT_REFLECTIONS_PAN_XYZ', 'AL_EAXREVERB_MIN_LATE_REVERB_GAIN',
'AL_EAXREVERB_MAX_LATE_REVERB_GAIN', 'AL_EAXREVERB_DEFAULT_LATE_REVERB_GAIN',
'AL_EAXREVERB_MIN_LATE_REVERB_DELAY', 'AL_EAXREVERB_MAX_LATE_REVERB_DELAY',
'AL_EAXREVERB_DEFAULT_LATE_REVERB_DELAY',
'AL_EAXREVERB_DEFAULT_LATE_REVERB_PAN_XYZ',
'AL_EAXREVERB_MIN_ECHO_TIME', 'AL_EAXREVERB_MAX_ECHO_TIME',
'AL_EAXREVERB_DEFAULT_ECHO_TIME', 'AL_EAXREVERB_MIN_ECHO_DEPTH',
'AL_EAXREVERB_MAX_ECHO_DEPTH', 'AL_EAXREVERB_DEFAULT_ECHO_DEPTH',
'AL_EAXREVERB_MIN_MODULATION_TIME', 'AL_EAXREVERB_MAX_MODULATION_TIME',
'AL_EAXREVERB_DEFAULT_MODULATION_TIME', 'AL_EAXREVERB_MIN_MODULATION_DEPTH',
'AL_EAXREVERB_MAX_MODULATION_DEPTH', 'AL_EAXREVERB_DEFAULT_MODULATION_DEPTH',
'AL_EAXREVERB_MIN_AIR_ABSORPTION_GAINHF',
'AL_EAXREVERB_MAX_AIR_ABSORPTION_GAINHF',
'AL_EAXREVERB_DEFAULT_AIR_ABSORPTION_GAINHF',
'AL_EAXREVERB_MIN_HFREFERENCE', 'AL_EAXREVERB_MAX_HFREFERENCE',
'AL_EAXREVERB_DEFAULT_HFREFERENCE', 'AL_EAXREVERB_MIN_LFREFERENCE',
'AL_EAXREVERB_MAX_LFREFERENCE', 'AL_EAXREVERB_DEFAULT_LFREFERENCE',
'AL_EAXREVERB_MIN_ROOM_ROLLOFF_FACTOR', 'AL_EAXREVERB_MAX_ROOM_ROLLOFF_FACTOR',
'AL_EAXREVERB_DEFAULT_ROOM_ROLLOFF_FACTOR', 'AL_EAXREVERB_MIN_DECAY_HFLIMIT',
'AL_EAXREVERB_MAX_DECAY_HFLIMIT', 'AL_EAXREVERB_DEFAULT_DECAY_HFLIMIT',
'AL_CHORUS_WAVEFORM_SINUSOID', 'AL_CHORUS_WAVEFORM_TRIANGLE',
'AL_CHORUS_MIN_WAVEFORM', 'AL_CHORUS_MAX_WAVEFORM', 'AL_CHORUS_DEFAULT_WAVEFORM',
'AL_CHORUS_MIN_PHASE', 'AL_CHORUS_MAX_PHASE', 'AL_CHORUS_DEFAULT_PHASE', 
'AL_CHORUS_MIN_RATE', 'AL_CHORUS_MAX_RATE', 'AL_CHORUS_DEFAULT_RATE',
'AL_CHORUS_MIN_DEPTH', 'AL_CHORUS_MAX_DEPTH', 'AL_CHORUS_DEFAULT_DEPTH', 
'AL_CHORUS_MIN_FEEDBACK', 'AL_CHORUS_MAX_FEEDBACK', 'AL_CHORUS_DEFAULT_FEEDBACK',
'AL_CHORUS_MIN_DELAY', 'AL_CHORUS_MAX_DELAY', 'AL_CHORUS_DEFAULT_DELAY',
'AL_DISTORTION_MIN_EDGE', 'AL_DISTORTION_MAX_EDGE', 'AL_DISTORTION_DEFAULT_EDGE',
'AL_DISTORTION_MIN_GAIN', 'AL_DISTORTION_MAX_GAIN', 'AL_DISTORTION_DEFAULT_GAIN',
'AL_DISTORTION_MIN_LOWPASS_CUTOFF', 'AL_DISTORTION_MAX_LOWPASS_CUTOFF',
'AL_DISTORTION_DEFAULT_LOWPASS_CUTOFF', 'AL_DISTORTION_MIN_EQCENTER',
'AL_DISTORTION_MAX_EQCENTER', 'AL_DISTORTION_DEFAULT_EQCENTER',
'AL_DISTORTION_MIN_EQBANDWIDTH', 'AL_DISTORTION_MAX_EQBANDWIDTH',
'AL_DISTORTION_DEFAULT_EQBANDWIDTH', 'AL_ECHO_MIN_DELAY', 'AL_ECHO_MAX_DELAY',
'AL_ECHO_DEFAULT_DELAY', 'AL_ECHO_MIN_LRDELAY', 'AL_ECHO_MAX_LRDELAY',
'AL_ECHO_DEFAULT_LRDELAY', 'AL_ECHO_MIN_DAMPING', 'AL_ECHO_MAX_DAMPING',
'AL_ECHO_DEFAULT_DAMPING', 'AL_ECHO_MIN_FEEDBACK', 'AL_ECHO_MAX_FEEDBACK',
'AL_ECHO_DEFAULT_FEEDBACK', 'AL_ECHO_MIN_SPREAD', 'AL_ECHO_MAX_SPREAD',
'AL_ECHO_DEFAULT_SPREAD', 'AL_FLANGER_WAVEFORM_SINUSOID',
'AL_FLANGER_WAVEFORM_TRIANGLE', 'AL_FLANGER_MIN_WAVEFORM',
'AL_FLANGER_MAX_WAVEFORM', 'AL_FLANGER_DEFAULT_WAVEFORM', 'AL_FLANGER_MIN_PHASE',
'AL_FLANGER_MAX_PHASE', 'AL_FLANGER_DEFAULT_PHASE', 'AL_FLANGER_MIN_RATE',
'AL_FLANGER_MAX_RATE', 'AL_FLANGER_DEFAULT_RATE', 'AL_FLANGER_MIN_DEPTH',
'AL_FLANGER_MAX_DEPTH', 'AL_FLANGER_DEFAULT_DEPTH', 'AL_FLANGER_MIN_FEEDBACK',
'AL_FLANGER_MAX_FEEDBACK', 'AL_FLANGER_DEFAULT_FEEDBACK', 'AL_FLANGER_MIN_DELAY',
'AL_FLANGER_MAX_DELAY', 'AL_FLANGER_DEFAULT_DELAY',
'AL_FREQUENCY_SHIFTER_MIN_FREQUENCY', 'AL_FREQUENCY_SHIFTER_MAX_FREQUENCY',
'AL_FREQUENCY_SHIFTER_DEFAULT_FREQUENCY',
'AL_FREQUENCY_SHIFTER_MIN_LEFT_DIRECTION',
'AL_FREQUENCY_SHIFTER_MAX_LEFT_DIRECTION',
'AL_FREQUENCY_SHIFTER_DEFAULT_LEFT_DIRECTION',
'AL_FREQUENCY_SHIFTER_DIRECTION_DOWN', 'AL_FREQUENCY_SHIFTER_DIRECTION_UP',
'AL_FREQUENCY_SHIFTER_DIRECTION_OFF',
'AL_FREQUENCY_SHIFTER_MIN_RIGHT_DIRECTION',
'AL_FREQUENCY_SHIFTER_MAX_RIGHT_DIRECTION',
'AL_FREQUENCY_SHIFTER_DEFAULT_RIGHT_DIRECTION',
'AL_VOCAL_MORPHER_MIN_PHONEMEA', 'AL_VOCAL_MORPHER_MAX_PHONEMEA',
'AL_VOCAL_MORPHER_DEFAULT_PHONEMEA',
'AL_VOCAL_MORPHER_MIN_PHONEMEA_COARSE_TUNING',
'AL_VOCAL_MORPHER_MAX_PHONEMEA_COARSE_TUNING',
'AL_VOCAL_MORPHER_DEFAULT_PHONEMEA_COARSE_TUNING',
'AL_VOCAL_MORPHER_MIN_PHONEMEB', 'AL_VOCAL_MORPHER_MAX_PHONEMEB',
'AL_VOCAL_MORPHER_DEFAULT_PHONEMEB',
'AL_VOCAL_MORPHER_MIN_PHONEMEB_COARSE_TUNING',
'AL_VOCAL_MORPHER_MAX_PHONEMEB_COARSE_TUNING',
'AL_VOCAL_MORPHER_DEFAULT_PHONEMEB_COARSE_TUNING',
'AL_VOCAL_MORPHER_PHONEME_A', 'AL_VOCAL_MORPHER_PHONEME_E',
'AL_VOCAL_MORPHER_PHONEME_I', 'AL_VOCAL_MORPHER_PHONEME_O',
'AL_VOCAL_MORPHER_PHONEME_U', 'AL_VOCAL_MORPHER_PHONEME_AA',
'AL_VOCAL_MORPHER_PHONEME_AE', 'AL_VOCAL_MORPHER_PHONEME_AH',
'AL_VOCAL_MORPHER_PHONEME_AO', 'AL_VOCAL_MORPHER_PHONEME_EH',
'AL_VOCAL_MORPHER_PHONEME_ER', 'AL_VOCAL_MORPHER_PHONEME_IH',
'AL_VOCAL_MORPHER_PHONEME_IY', 'AL_VOCAL_MORPHER_PHONEME_UH',
'AL_VOCAL_MORPHER_PHONEME_UW', 'AL_VOCAL_MORPHER_PHONEME_B',
'AL_VOCAL_MORPHER_PHONEME_D', 'AL_VOCAL_MORPHER_PHONEME_F',
'AL_VOCAL_MORPHER_PHONEME_G', 'AL_VOCAL_MORPHER_PHONEME_J',
'AL_VOCAL_MORPHER_PHONEME_K', 'AL_VOCAL_MORPHER_PHONEME_L',
'AL_VOCAL_MORPHER_PHONEME_M', 'AL_VOCAL_MORPHER_PHONEME_N',
'AL_VOCAL_MORPHER_PHONEME_P', 'AL_VOCAL_MORPHER_PHONEME_R',
'AL_VOCAL_MORPHER_PHONEME_S', 'AL_VOCAL_MORPHER_PHONEME_T',
'AL_VOCAL_MORPHER_PHONEME_V', 'AL_VOCAL_MORPHER_PHONEME_Z',
'AL_VOCAL_MORPHER_WAVEFORM_SINUSOID', 'AL_VOCAL_MORPHER_WAVEFORM_TRIANGLE',
'AL_VOCAL_MORPHER_WAVEFORM_SAWTOOTH', 'AL_VOCAL_MORPHER_MIN_WAVEFORM',
'AL_VOCAL_MORPHER_MAX_WAVEFORM', 'AL_VOCAL_MORPHER_DEFAULT_WAVEFORM',
'AL_VOCAL_MORPHER_MIN_RATE', 'AL_VOCAL_MORPHER_MAX_RATE',
'AL_VOCAL_MORPHER_DEFAULT_RATE', 'AL_PITCH_SHIFTER_MIN_COARSE_TUNE',
'AL_PITCH_SHIFTER_MAX_COARSE_TUNE', 'AL_PITCH_SHIFTER_DEFAULT_COARSE_TUNE',
'AL_PITCH_SHIFTER_MIN_FINE_TUNE', 'AL_PITCH_SHIFTER_MAX_FINE_TUNE',
'AL_PITCH_SHIFTER_DEFAULT_FINE_TUNE', 'AL_RING_MODULATOR_MIN_FREQUENCY',
'AL_RING_MODULATOR_MAX_FREQUENCY', 'AL_RING_MODULATOR_DEFAULT_FREQUENCY',
'AL_RING_MODULATOR_MIN_HIGHPASS_CUTOFF',
'L_RING_MODULATOR_MAX_HIGHPASS_CUTOFF',
'L_RING_MODULATOR_DEFAULT_HIGHPASS_CUTOFF',
'AL_RING_MODULATOR_SINUSOID', 'AL_RING_MODULATOR_SAWTOOTH',
'AL_RING_MODULATOR_SQUARE', 'AL_RING_MODULATOR_MIN_WAVEFORM',
'AL_RING_MODULATOR_MAX_WAVEFORM', 'AL_RING_MODULATOR_DEFAULT_WAVEFORM',
'AL_AUTOWAH_MIN_ATTACK_TIME', 'AL_AUTOWAH_MAX_ATTACK_TIME',
'AL_AUTOWAH_DEFAULT_ATTACK_TIME', 'AL_AUTOWAH_MIN_RELEASE_TIME',
'AL_AUTOWAH_MAX_RELEASE_TIME', 'AL_AUTOWAH_DEFAULT_RELEASE_TIME',
'AL_AUTOWAH_MIN_RESONANCE', 'AL_AUTOWAH_MAX_RESONANCE',
'AL_AUTOWAH_DEFAULT_RESONANCE', 'AL_AUTOWAH_MIN_PEAK_GAIN',
'AL_AUTOWAH_MAX_PEAK_GAIN', 'AL_AUTOWAH_DEFAULT_PEAK_GAIN',
'AL_COMPRESSOR_MIN_ONOFF', 'AL_COMPRESSOR_MAX_ONOFF',
'AL_COMPRESSOR_DEFAULT_ONOFF', 'AL_EQUALIZER_MIN_LOW_GAIN',
'AL_EQUALIZER_MAX_LOW_GAIN', 'AL_EQUALIZER_DEFAULT_LOW_GAIN',
'AL_EQUALIZER_MIN_LOW_CUTOFF', 'AL_EQUALIZER_MAX_LOW_CUTOFF',
'AL_EQUALIZER_DEFAULT_LOW_CUTOFF', 'AL_EQUALIZER_MIN_MID1_GAIN',
'AL_EQUALIZER_MAX_MID1_GAIN', 'AL_EQUALIZER_DEFAULT_MID1_GAIN',
'AL_EQUALIZER_MIN_MID1_CENTER', 'AL_EQUALIZER_MAX_MID1_CENTER',
'AL_EQUALIZER_DEFAULT_MID1_CENTER', 'AL_EQUALIZER_MIN_MID1_WIDTH',
'AL_EQUALIZER_MAX_MID1_WIDTH', 'AL_EQUALIZER_DEFAULT_MID1_WIDTH',
'AL_EQUALIZER_MIN_MID2_GAIN', 'AL_EQUALIZER_MAX_MID2_GAIN',
'AL_EQUALIZER_DEFAULT_MID2_GAIN', 'AL_EQUALIZER_MIN_MID2_CENTER',
'AL_EQUALIZER_MAX_MID2_CENTER', 'AL_EQUALIZER_DEFAULT_MID2_CENTER',
'AL_EQUALIZER_MIN_MID2_WIDTH', 'AL_EQUALIZER_MAX_MID2_WIDTH',
'AL_EQUALIZER_DEFAULT_MID2_WIDTH', 'AL_EQUALIZER_MIN_HIGH_GAIN',
'AL_EQUALIZER_MAX_HIGH_GAIN', 'AL_EQUALIZER_DEFAULT_HIGH_GAIN',
'AL_EQUALIZER_MIN_HIGH_CUTOFF', 'AL_EQUALIZER_MAX_HIGH_CUTOFF',
'AL_EQUALIZER_DEFAULT_HIGH_CUTOFF', 'AL_MIN_AIR_ABSORPTION_FACTOR',
'AL_MAX_AIR_ABSORPTION_FACTOR', 'AL_DEFAULT_AIR_ABSORPTION_FACTOR',
'AL_MIN_ROOM_ROLLOFF_FACTOR', 'AL_MAX_ROOM_ROLLOFF_FACTOR',
'AL_DEFAULT_ROOM_ROLLOFF_FACTOR', 'AL_MIN_CONE_OUTER_GAINHF',
'AL_MAX_CONE_OUTER_GAINHF', 'AL_DEFAULT_CONE_OUTER_GAINHF',
'AL_MIN_DIRECT_FILTER_GAINHF_AUTO', 'AL_MAX_DIRECT_FILTER_GAINHF_AUTO',
'AL_DEFAULT_DIRECT_FILTER_GAINHF_AUTO',
'AL_MIN_AUXILIARY_SEND_FILTER_GAIN_AUTO',
'AL_MAX_AUXILIARY_SEND_FILTER_GAIN_AUTO',
'AL_DEFAULT_AUXILIARY_SEND_FILTER_GAIN_AUTO',
'AL_MIN_AUXILIARY_SEND_FILTER_GAINHF_AUTO',
'AL_MAX_AUXILIARY_SEND_FILTER_GAINHF_AUTO',
'AL_DEFAULT_AUXILIARY_SEND_FILTER_GAINHF_AUTO',
'AL_MIN_METERS_PER_UNIT', 'AL_MAX_METERS_PER_UNIT',
'AL_DEFAULT_METERS_PER_UNIT', 'alGenEffects', 'alDeleteEffects', 'alIsEffect',
'alEffecti', 'alEffectiv', 'alEffectf', 'alEffectfv', 'alGetEffecti',
'alGetEffectiv', 'alGetEffectf', 'alGetEffectfv', 'alGenFilters',
'alDeleteFilters', 'alIsFilter', 'alFilteri', 'alFilteriv', 'alFilterf',
'alFilterfv', 'alGetFilteri', 'alGetFilteriv', 'alGetFilterf', 'alGetFilterfv',
'alGenAuxiliaryEffectSlots', 'alDeleteAuxiliaryEffectSlots',
'alIsAuxiliaryEffectSlot', 'alAuxiliaryEffectSloti',
'alAuxiliaryEffectSlotiv', 'alAuxiliaryEffectSlotf',
'alAuxiliaryEffectSlotfv', 'alGetAuxiliaryEffectSloti',
'alGetAuxiliaryEffectSlotiv', 'alGetAuxiliaryEffectSlotf',
'alGetAuxiliaryEffectSlotfv']
